<?php
/**
 * @file file is used for comparing entity data and preparing them for logging
 */

function entity_log_compare($entity,$type,$log_type,$fields = FALSE){
  $log_data = [
    'properties' => [],
    'fields' => [],
  ];
  if(isset($fields['fields'])){
    //prepare data from fields
    foreach($fields['fields'] as $field_key => $field){
        try{
          $entity_wrapper_old = entity_metadata_wrapper($type, $entity->original);
          $entity_wrapper_new = entity_metadata_wrapper($type, $entity);
          $field_info = field_info_field($field_key);
          $field_type = $field_info['type'];
          module_load_include('inc','entity_log','includes/entity_log.fields');
          switch ($field_type) {
            case 'text_with_summary':
              if($log_data_new = entity_log_field_text_with_summary($entity_wrapper_old, $entity_wrapper_new, $field)){
                $log_data['fields'] = array_merge($log_data['fields'], $log_data_new);
              }
              break;
            case 'text':
            case 'text_long':
              if($log_data_new = entity_log_field_text($entity_wrapper_old, $entity_wrapper_new, $field)){
                $log_data['fields'] = array_merge($log_data['fields'], $log_data_new);
              }
              break;
            case 'file':
              if($log_data_new = entity_log_field_file($entity_wrapper_old, $entity_wrapper_new, $field)){
                $log_data['fields'] = array_merge($log_data['fields'], $log_data_new);
              }
              break;
            case 'image':
              if($log_data_new = entity_log_field_image($entity_wrapper_old, $entity_wrapper_new, $field)){
                $log_data['fields'] = array_merge($log_data['fields'], $log_data_new);
              }
              break;

            case 'taxonomy_term_reference':
              if($log_data_new = entity_log_field_taxonomy_term_reference($entity_wrapper_old, $entity_wrapper_new, $field)){
                $log_data['fields'] = array_merge($log_data['fields'], $log_data_new);
              }
              break;
            case 'list_float':
            case 'list_integer':
            case 'list_text':
            case 'list_boolean':
              if($log_data_new = entity_log_field_list($entity_wrapper_old, $entity_wrapper_new, $field)){
                $log_data['fields'] = array_merge($log_data['fields'], $log_data_new);
              }
              break;
            case 'number_decimal':
            case 'number_float':
            case 'number_integer':
              if($log_data_new = entity_log_field_number($entity_wrapper_old, $entity_wrapper_new, $field)){
                $log_data['fields'] = array_merge($log_data['fields'], $log_data_new);
              }
              break;
            case 'datetime':
            case 'datestamp':
            case 'date':
              if($log_data_new = entity_log_field_datetime($entity_wrapper_old, $entity_wrapper_new, $field)){
                $log_data['fields'] = array_merge($log_data['fields'], $log_data_new);
              }
            break;
            case 'commerce_price':
              if($log_data_new = entity_log_field_commerce_price($entity_wrapper_old, $entity_wrapper_new, $field)){
                $log_data['fields'] = array_merge($log_data['fields'], $log_data_new);
              }
            break;
            case 'commerce_line_item_reference':
              if($log_data_new = entity_log_field_commerce_line_item_reference($entity_wrapper_old, $entity_wrapper_new, $field)){
                $log_data['fields'] = array_merge($log_data['fields'], $log_data_new);
              }
            break;
            default:
              if(isset($entity_wrapper_old->$field->value)){
                if($entity_wrapper_old->$field->value() != $entity_wrapper_new->$field->value()){
                  $old_value = $entity_wrapper_old->$field->value();
                  $new_value = $entity_wrapper_new->$field->value();
                  if(is_array($old_value)){
                    $old_value = implode($old_value,',');
                  }elseif(is_object($old_value)){
                    $old_value = (array)$old_value;
                    $old_value = implode($old_value,',');
                  }
                  if(is_array($new_value)){
                    $new_value = implode($new_value,',');
                  }elseif(is_object($new_value)){
                    $new_value = (array)$new_value;
                    $new_value = implode($new_value,',');
                  }
                  $log_data['fields'][$field]['old_value'] = $old_value;
                  $log_data['fields'][$field]['new_value'] = $new_value;
                }
              }elseif($entity_wrapper_old->$field->value() != $entity_wrapper_new->$field->value()){
                //if fields are not same then log this
                $log_data['fields'][$field]['old_value'] = $entity_wrapper_old->$field->value();
                $log_data['fields'][$field]['new_value'] = $entity_wrapper_new->$field->value();
              }
              break;
          }


        }catch (EntityMetadataWrapperException $exc) {
          watchdog(
            'MODULE_NAME',
            'EntityMetadataWrapper exception in %function() @trace',
            array('%function' => __FUNCTION__, '@trace' => $exc->getTraceAsString()),
            WATCHDOG_ERROR
          );
        }
    }
  }
  if(isset($fields['properties'])){
    //prepare data from properties
    foreach($fields['properties'] as $property){
      //@todo compare field from original and current
      if($entity->original->$property != $entity->$property){
        //if fields are not same then log this
        $log_data['properties'][$property]['old_value'] = $entity->original->$property;
        $log_data['properties'][$property]['new_value'] = $entity->$property;
      }
    }
  }
  return $log_data;
}
